@@ -87,6 +87,10 @@ static void oobs_object_get_property (GObject       *object,
 				      GValue        *value,
 				      GParamSpec    *pspec);
 
+static DBusHandlerResult changed_signal_filter (DBusConnection *connection,
+                                                DBusMessage    *message,
+                                                void           *user_data);
+
 static void connect_object_to_session (OobsObject *object);
 
 enum
@@ -119,9 +123,6 @@ oobs_object_class_init (OobsObjectClass *class)
   object_class->set_property = oobs_object_set_property;
   object_class->finalize     = oobs_object_finalize;
 
-  /* some object types don't need to be singletons, they override this */
-  class->singleton = TRUE;
-
   dbus_connection_quark = g_quark_from_static_string ("oobs-dbus-connection");
 
   g_object_class_install_property (object_class,
@@ -179,6 +180,7 @@ oobs_object_finalize (GObject *object)
 {
   OobsObject *obj;
   OobsObjectPrivate *priv;
+  DBusConnection *connection;
 
   g_return_if_fail (OOBS_IS_OBJECT (object));
 
@@ -190,11 +192,10 @@ oobs_object_finalize (GObject *object)
   g_list_foreach (priv->pending_calls, (GFunc) dbus_pending_call_unref, NULL);
   g_list_free (priv->pending_calls);
 
-  _oobs_session_unregister_object (priv->session, obj);
+  connection = _oobs_session_get_connection_bus (priv->session);
+  dbus_connection_remove_filter (connection, changed_signal_filter, object);
 
-  /* Only now, we don't care about session */
   g_object_unref (priv->session);
-
   g_free (priv->remote_object);
   g_free (priv->path);
   g_free (priv->method);
@@ -208,32 +209,12 @@ oobs_object_constructor (GType                  type,
 			 guint                  n_construct_properties,
 			 GObjectConstructParam *construct_params)
 {
-  OobsObjectClass *class;
-  static GHashTable *type_table;
   GObject *object;
 
-  /* some objects that inherit from OobsObject have to be
-   * singletons, keep a hash table with references to them
-   */
-  if (!type_table)
-    type_table = g_hash_table_new (g_direct_hash, g_direct_equal);
-
-  object = g_hash_table_lookup (type_table, GINT_TO_POINTER (type));
-
-  if (object)
-    {
-      class = OOBS_OBJECT_GET_CLASS (object);
-
-      /* if has to be singleton and exists, return it */
-      if (class->singleton)
-	return object;
-    }
-
-  /* this class should not be limited to singletons, create (possibly new) object anyway */
   object = (* G_OBJECT_CLASS (oobs_object_parent_class)->constructor) (type,
                                                                        n_construct_properties,
                                                                        construct_params);
-  g_hash_table_insert (type_table, GINT_TO_POINTER (type), object);
+
   connect_object_to_session (OOBS_OBJECT (object));
 
   return object;
@@ -288,7 +269,6 @@ connect_object_to_session (OobsObject *object)
       return;
     }
 
-  _oobs_session_register_object (priv->session, object);
   dbus_connection_add_filter (connection, changed_signal_filter, object, NULL);
 
   rule = g_strdup_printf ("type='signal',interface='%s',path='%s'",
